//
//  ALNetClient
//
//  gm_alpha_flutter
//  Created by lxrent on 2019/1/31.
//  Copyright © 2019 Gengmei. All rights reserved.
//

import 'dart:convert';
import 'dart:async';
import 'ALServers.dart';
import 'package:dio/dio.dart';
import 'dart:io';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:device_info/device_info.dart';

export 'ALServers.dart';

class ALResponse {
  const ALResponse(
      {this.isSuccess,
        this.data,
        this.error,
        this.errorCode,
        this.message});

  final bool isSuccess;
  final dynamic data;
  final int error;
  final String errorCode;
  final String message;

  factory ALResponse.fromResponse(Response response) {
    if(response.statusCode == 403) return null;

    var data = jsonDecode(response.toString());

    print('${response.request.path}\n${data}\n${response.request.cookies}');
    return new ALResponse(
      isSuccess: data['error'] == 0?true:false,
      data: data,
      error: data['error'],
      errorCode: data['errorCode'],
      message: data['message'],
    );
  }
}

// 通用参数
String urlCommonParameters() {
  DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();

  return 'platform=iPhone&os_version=12.1.2&version=1.3.0&model=iPhone%20XS%20Max&release=0&idfa=EE10F836-B882-4AB8-9AB1-8E116BD48DEE&idfv=178AB378-66BF-46B3-BD12-C10FAA35C74C&device_id=EE10F836-B882-4AB8-9AB1-8E116BD48DEE&channel=(null)&app_name=gengmeiios&current_city_id=worldwide&lat=0&lng=0&is_WiFi=(null)&phone_id=iPhone11,6&count=10&page=1&type=1';
}

// 全链接
String fullAPI(api) {
  return apiHost() + api + '?' + urlCommonParameters();
}

// request请求
Options requestOptions(type) {

  Options options = new Options(
      contentType: new ContentType('application', 'x-www-form-urlencoded',charset: 'utf-8'),
      headers: {'User-Agent':'Mozilla/5.0 (iPhone; CPU iPhone OS 10_10_1 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Mobile/12B411 GMAlpha/1.2.1'},
      connectTimeout: ALServers.ConnetTimeOut,
      receiveTimeout: ALServers.ReceiveTimeOut,
      method: type,
  );
  return options;
}

Future<ALResponse> excuteGet(url, params) async {

  Dio dio = new Dio();
  SharedPreferences pre = await SharedPreferences.getInstance();
  Options options = requestOptions('GET');
  options.cookies = createGMTokenCookie(pre.get(apiHost()));

  var response = await dio
      .request(fullAPI(url), data: params, options: options);
  return await ALResponse.fromResponse(response);

}

List<Cookie> createGMTokenCookie(List<String> cookieString) {

  List<Cookie> cookieList = List();
  for(String value in cookieString) {

    Cookie cookie = Cookie.fromSetCookieValue(value);
    cookieList.add(cookie);
  }
  Cookie gm_token = Cookie.fromSetCookieValue('_gm_token=decf991549869501');
  cookieList.add(gm_token);
  return cookieList;
}

Future<ALResponse> excutePost(url, params) async {


  Dio dio = new Dio();
  SharedPreferences pre = await SharedPreferences.getInstance();
  Options options = requestOptions('POST');
  options.cookies = createGMTokenCookie(pre.get(apiHost()));
  var response = await dio
      .request(fullAPI(url), data: params, options: options);
  if(response.headers['Set-Cookie'] != null) {
    // 保存cookie
    List cookie = response.headers[HttpHeaders.setCookieHeader];
    pre.setStringList(apiHost(), cookie);
  }
  return await ALResponse.fromResponse(response);
}
